#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import os.path
import gettext
import logging

import ecore
import elementary as elm
from evas import EVAS_ASPECT_CONTROL_VERTICAL, EVAS_HINT_FILL, EVAS_HINT_EXPAND

from Globals import *
from AccountEditor import AccountEditor
from PresenceEditor import PresenceEditor

from gi.repository import GObject
GObject.threads_init()
from gi.repository import TelepathyGLib as Tp

class Accounts(object):
    def __init__(self):
        self.accounts = {}
        self._signals = []
        self.accounts_list_items = {}

        self.window = window = elm.StandardWindow("accounts", "Apathy - Accounts")
        window.callback_delete_request_add(self.quit)

        box = elm.Box(window)

        self.accounts_list = accounts_list = elm.Genlist(window)
        accounts_list.size_hint_weight = (EVAS_HINT_EXPAND, EVAS_HINT_EXPAND)
        accounts_list.size_hint_align = (EVAS_HINT_FILL, EVAS_HINT_FILL)
        accounts_list.mode_set(elm.ELM_LIST_COMPRESS)
        accounts_list.callback_clicked_double_add(self._edit_account)
        accounts_list.callback_selected_add(self._selected)
        accounts_list.show()
        box.pack_end(accounts_list)

        self.pb = pb = elm.Progressbar(window)
        pb.text = _("Loading accounts")
        pb.size_hint_weight = (EVAS_HINT_EXPAND, 0.0)
        pb.size_hint_align = (EVAS_HINT_FILL, 0.0)
        box.pack_end(pb)
        pb.show()

        edit_box = elm.Box(window)
        edit_box.horizontal = True
        edit_box.size_hint_weight = (EVAS_HINT_EXPAND, 0.0)
        edit_box.size_hint_align = (EVAS_HINT_FILL, 1.0)

        for label, func in {_("Add"): self._add_account, _("Delete"): self._delete_account}.iteritems():
            btn = elm.Button(window)
            btn.callback_clicked_add(func)
            btn.size_hint_weight = (EVAS_HINT_EXPAND, 0.0)
            btn.size_hint_align = (EVAS_HINT_FILL, 0.0)
            btn.text = label
            btn.show()
            edit_box.pack_end(btn)

        self.presence_list = presence_list = elm.Hoversel(window)
        presence_list.text = _("Set presence").encode("utf-8")
        self._populate_presence_list()
        presence_list.show()
        edit_box.pack_end(presence_list)

        box.pack_end(edit_box)
        edit_box.show()

        box.show()

        self.pager = pager = elm.Naviframe(window)
        pager.size_hint_weight = (EVAS_HINT_EXPAND, EVAS_HINT_EXPAND)
        pager.size_hint_align = (EVAS_HINT_FILL, EVAS_HINT_FILL)
        window.resize_object_add(pager)
        pager.item_simple_push(box)
        pager.show()

        window.resize(480, 640)
        window.show()

        self.am = am = Tp.AccountManager.dup()
        factory = am.get_factory()
        factory.add_account_features([Tp.Account.get_feature_quark_connection()])
        factory.add_connection_features([
            Tp.Connection.get_feature_quark_contact_list(),
            Tp.Connection.get_feature_quark_contact_groups(),
            Tp.Connection.get_feature_quark_avatar_requirements(),
        ])

        am.prepare_async(None, self._am_ready, None)

    def _am_ready(self, am, result, error):
        am.connect("account-enabled", self._account_enabled)
        am.connect("account-disabled", self._account_disabled)
        am.connect("account-removed", self._account_removed)
        am.connect("account-validity-changed", self._account_validity_changed)
        accounts = am.get_valid_accounts()
        self.progress = 1.0
        if len(accounts) == 0:
            self.pb.delete()
            del self.pb
        else:
            self.progress_unit = 1.0/len(accounts)
        for account in accounts:
            account.prepare_async(None, self._account_ready, None)

    def _account_ready(self, account, result, error):
        try: self.pb
        except:
            pass
        else:
            self.progress = self.progress - self.progress_unit
            self.pb.value = self.progress
            if self.progress <= 0.01:
                self.pb.delete()
                del self.pb
        item_class = AccountItem("default")
        self.accounts_list_items[account.get_path_suffix()] = self.accounts_list.item_append(item_class, account, None, elm.ELM_GENLIST_ITEM_NONE)
        account.connect("presence-changed", self._presence_changed)
        account.connect("notify::changing-presence", self._presence_changing)

    def _account_enabled(self, am, account, *args):
        pass #self.accounts_list_items[account.get_path_suffix()].disabled_set(False)

    def _account_disabled(self, am, account, *args):
        pass #self.accounts_list_items[account.get_path_suffix()].disabled_set(True)

    def _account_removed(self, am, account, *args):
        self.accounts_list_items[account.get_path_suffix()].delete()

    def _account_validity_changed(self, am, account, valid, *args):
        if valid:
            account.prepare_async(None, self._account_ready, None)

    def _populate_presence_list(self):
        presences = {
            "offline": (1, True, False),
            "available": (2, True, True),
            "away": (3, True, True),
            "xa": (4, True, True),
            "hidden": (5, True, False),
            "dnd": (6, True, True),
            "unknown": (7, False, False),
            "error": (8, False, False),
        }

        conf = get_conf()

        for status, spec in presences.iteritems():
            presence, settable_on_self, can_have_message = spec
            if settable_on_self:
                if can_have_message:
                    try:
                        message = conf.get("StatusMessages", status)
                    except:
                        logging.exception("Status message for status %s not found in configuration file." % (status))
                        message = ""
                    self.presence_list.item_add(status.capitalize(), None, 0, self._set_presence, presence, status, message)
                else:
                    self.presence_list.item_add(status.capitalize(), None, 0, self._set_presence, presence, status)

        self.presence_list.item_add(_("Edit")+"...", None, 0, self._edit_presences)

    def _edit_presences(self, obj, event, *args):
        editor = PresenceEditor(self.window, self.pager)
        self.pager.item_simple_push(editor.box)

    def _add_account(self, obj, *args):
        editor = AccountEditor(self.window, self.pager, self.am)
        self.pager.item_simple_push(editor.box)

    def _edit_account(self, genlist, genlistitem, *args):
        account = genlistitem.data_get()
        editor = AccountEditor(self.window, self.pager, self.am, account)
        self.pager.item_simple_push(editor.box)

    def _delete_account(self, obj, *args):
        if(self.accounts_list.selected_item):
            account = self.accounts_list.selected_item.data
            account.remove_async(self._account_deleted(), None)

    def _set_presence(self, obj, event, prestype, presence, message=""):
        if self.accounts_list.selected_item:
            account = self.accounts_list.selected_item.data
            logging.info("Setting presence %s for account %s" % (presence, account.get_display_name()))
            account.request_presence_async(prestype, presence, message, self._changed_presence, None)
            obj.text = _("Set presence")+" ("+presence+")"

    def _changed_presence(self, account, result, *args):
        try:
            account.request_presence_finish(result)
        except:
            logging.exception("Setting presence failed")
        else:
            logging.info("Presence set")

    def _account_deleted(self):
        logging.info("Account deleted")

    def _presence_changed(self, account, presence, status, status_message):
        self.accounts_list_items[account.get_path_suffix()].update()

    def _presence_changing(self, account, changing):
        self.accounts_list_items[account.get_path_suffix()].update()

    def _balance_changed(self, *args):
        pass

    def _selected(self, obj, item):
        account = item.data_get()
        presence, status, status_message = account.get_requested_presence()
        self.presence_list.text_set(_("Set presence")+" ("+status.capitalize()+")")

    def quit(self, *args):
        elm.exit()

class AccountItem(elm.GenlistItemClass):
    def text_get(self, obj, part, account):
        presence, status, status_message = account.get_current_presence()
        text = "%s" % (account.get_display_name())
        if status_message:
            text += " [%s]" % (status_message)
        return text

    def content_get(self, obj, part, account):
        if part == "elm.swallow.icon":
            ic = elm.Icon(obj)
            if account.get_icon_name():
                ic.file_set(os.path.join(IMAGE_PATH, account.get_icon_name()+".png"))
                ic.standard = account.get_icon_name()
            else:
                ic.standard = "apathetic"
        elif part == "elm.swallow.end":
            if account.get_changing_presence():
                ic = elm.Progressbar(obj)
                ic.style = "wheel"
                ic.pulse_set(True)
                ic.pulse(True)
            else:
                ic = elm.Icon(obj)
                presence, status, status_message = account.get_current_presence()
                ic.file_set(os.path.join(IMAGE_PATH, status+".png"))
                ic.standard = "user-"+status
        ic.size_hint_aspect = (EVAS_ASPECT_CONTROL_VERTICAL, 1, 1)
        return ic

if __name__ == "__main__":
    gettext.install("apathy", "/usr/share/locale", unicode=1)

    # Log
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    logging.basicConfig(format="[%(levelname)s] in %(module)s line %(lineno)s (%(funcName)s): %(message)s", level=logging.DEBUG)

    elm.init()

    ecore.main_loop_glib_integrate()
    accounts = Accounts()

    elm.run()
    elm.shutdown()
