#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import os.path
import gettext
import logging

import ecore
import elementary as elm
from evas import EVAS_ASPECT_CONTROL_VERTICAL

from Globals import *

from gi.repository import TelepathyGLib as Tp

class Approver(object):
    def __init__(self):
        global log
        log = logging.getLogger("apathy.approver")

        daemon = Tp.DBusDaemon.dup()
        factory = Tp.AutomaticClientFactory.new(daemon)
        factory.add_account_features([Tp.Account.get_feature_quark_connection()])
        factory.add_connection_features([
            Tp.Connection.get_feature_quark_contact_list(),
            Tp.Connection.get_feature_quark_contact_groups(),
            Tp.Connection.get_feature_quark_avatar_requirements(),
        ])
        factory.add_contact_features([
            Tp.ContactFeature.ALIAS,
            Tp.ContactFeature.CONTACT_GROUPS,
            Tp.ContactFeature.PRESENCE,
            Tp.ContactFeature.AVATAR_DATA,
            Tp.ContactFeature.SUBSCRIPTION_STATES,
            Tp.ContactFeature.CAPABILITIES,
            Tp.ContactFeature.CONTACT_INFO,
            Tp.ContactFeature.LOCATION,
        ])

        self.approver = Tp.SimpleApprover.new_with_factory(
            factory,                #Factory/am/tpdbus
            'Apathy.Approver',       #name
            False,                  #uniquify
            self.channel_approver,  #handler func
            None,                   #data
        )
        self.approver.add_approver_filter({
        })
        self.approver.register()

    def accept(self, button, dispatch_operation, win):
        dispatch_operation.handle_with_async(Tp.CLIENT_BUS_NAME_BASE+"Apathy.Handler", self.channel_handled, win)

    def cancelled(self, button, dispatch_operation, win):
        dispatch_operation.destroy_channels_async(Tp.ChannelGroupChangeReason.NONE, "", self.channel_dropped, win)

    def channel_handled(self, dispatch_operation, result, win):
        dispatch_operation.handle_with_finish(result)
        log.debug("%s %s %s" % (dispatch_operation, result, win))
        win.delete()

    def channel_dropped(self, dispatch_operation, result, win):
        dispatch_operation.destroy_channels_finish(result)
        log.debug("%s %s %s" % (dispatch_operation, result, win))
        win.delete()

    def channel_lost(self, channel, domain, code, message, user_data):
        log.debug("%s %s %s %s %s" % (channel, domain, code, message, user_data))

    def channel_approver(self, approver, account, connection, channels, dispatch_operation, context, user_data):
        log.debug("%s %s %s %s %s %s %s" % (approver, account, connection, channels, dispatch_operation, context, user_data))

        dispatch_operation.connect("channel-lost", self.channel_lost)

        window = elm.Window("approver", elm.ELM_WIN_DIALOG_BASIC)
        window.title = "Apathy - Approver"
        window.callback_delete_request_add(self.quit)

        bg = elm.Background(window)
        bg.size_hint_weight = 1, 1
        window.resize_object_add(bg)
        bg.show()

        box = elm.Box(window)
        box.size_hint_weight_set(1.0, 1.0)
        window.resize_object_add(box)
        box.show()

        title = elm.Label(window)
        title.text_set("<b>"+_("Accept channel from").encode("ascii", "xmlcharrefreplace")+"</b>")
        box.pack_end(title)
        title.show()

        for channel in channels:
            log.info("Asking approval for a channel from %s" % (channel.get_identifier()))

            identifier = channel.get_target_contact().get_alias()
            if not identifier:
                identifier = channel.get_identifier()
            label = elm.Label(window)
            label.size_hint_weight_set(1.0, 1.0)
            label.show()
            label.text_set("%s" % (identifier))
            box.pack_end(label)

        ok = elm.Button(window)
        ok.text_set(_("Yes").encode("utf-8"))
        ok_ic = elm.Icon(window)
        ok_ic.standard_set("call-start")
        ok.content_set(ok_ic)

        cancel = elm.Button(window)
        cancel.text_set(_("No").encode("utf-8"))
        cancel_ic = elm.Icon(window)
        cancel_ic.standard_set("call-stop")
        cancel.content_set(cancel_ic)

        ok.callback_clicked_add(self.accept, dispatch_operation, window)
        cancel.callback_clicked_add(self.cancelled, dispatch_operation, window)

        option_box = elm.Box(window)
        option_box.horizontal_set(True)
        option_box.size_hint_weight_set(1.0, 1.0)
        option_box.pack_end(ok)
        option_box.pack_end(cancel)
        ok.show()
        cancel.show()
        option_box.show()
        box.pack_end(option_box)

        window.show()

        context.accept()

    def quit(self, *args):
        elm.exit()

if __name__ == "__main__":
    gettext.install("apathy", "/usr/share/locale", unicode=1)

    # Log
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    log = logging.getLogger("apathy.approver")
    log.setLevel(logging.DEBUG)
    streamformatter = logging.Formatter("[%(levelname)s] in %(module)s line %(lineno)s (%(funcName)s): %(message)s")
    handler_stream = logging.StreamHandler()
    handler_stream.setFormatter(streamformatter)
    log.addHandler(handler_stream)

    elm.init()
    elm.policy_set(elm.ELM_POLICY_QUIT, elm.ELM_POLICY_QUIT_LAST_WINDOW_CLOSED)

    ecore.main_loop_glib_integrate()
    approver = Approver()

    elm.run()
    elm.shutdown()
