#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import os.path
import gettext
import logging

import ecore
import elementary as elm
from evas import EVAS_ASPECT_CONTROL_VERTICAL

from ContactInfo import ContactInfo
from Globals import *
from ErrorDialog import ErrorDialog

from gi.repository import GObject
GObject.threads_init()
from gi.repository import TelepathyGLib as Tp

class ContactItem(elm.GenlistItemClass):
    def text_get(self, obj, part, contact):
        if part == "elm.text":
            msg = contact.get_presence_message()
            if msg:
                string = " %s - %s" % (contact.get_alias(), contact.get_presence_message())
            else:
                string = " %s" % (contact.get_alias())
            return string
        if part == "elm.text.sub":
            return " %s" % contact.get_presence_message()

    def content_get(self, obj, part, contact):
        ic = elm.Icon(obj)
        if part == "elm.swallow.icon":
            if contact.get_presence_status() != "offline":
                avatar = contact.get_avatar_file()
                if avatar:
                    path = avatar.get_path()
                    ic.file_set(path)
                else:
                    ic.standard_set("apathetic")
        elif part == "elm.swallow.end":
            status = contact.get_presence_status()
            if status == "xa":
                status = "away-extended"
            ic.standard_set("user-"+status)
        ic.size_hint_aspect_set(EVAS_ASPECT_CONTROL_VERTICAL, 1, 1)

        return ic

class ContactGroup(elm.GenlistItemClass):
    def text_get(self, obj, part, contactgroup):
        return contactgroup
        
    def content_get(self, obj, part, contactgroup):
        if part == "elm.swallow.icon":
            ic = elm.Icon(obj)
            ic.standard_set("media-playback-start")
            ic.size_hint_aspect_set(EVAS_ASPECT_CONTROL_VERTICAL, 1, 1)
            return ic
        else:
            return

class Contacts(object):
    def __init__(self):
        self.accounts = {}
        self._signals = []

        self.contacts = {}
        self.contact_items = {}
        self.group_items = {}
        self.selected_contact = None

        global log
        log = logging.getLogger("apathy.contacts")
        
        self.window = window = elm.StandardWindow("contacts", "Apathy - Contacts")
        window.callback_delete_request_add(self.quit)

        self.am = am = Tp.AccountManager.dup()
        factory = am.get_factory()
        factory.add_account_features([Tp.Account.get_feature_quark_connection()])
        factory.add_connection_features([
            Tp.Connection.get_feature_quark_contact_list(),
            Tp.Connection.get_feature_quark_contact_groups(),
            Tp.Connection.get_feature_quark_contact_blocking(),
            Tp.Connection.get_feature_quark_avatar_requirements(),
        ])
        factory.add_contact_features([
            Tp.ContactFeature.ALIAS,
            Tp.ContactFeature.CONTACT_GROUPS,
            Tp.ContactFeature.PRESENCE,
            Tp.ContactFeature.AVATAR_DATA,
            Tp.ContactFeature.SUBSCRIPTION_STATES,
            Tp.ContactFeature.CAPABILITIES,
            Tp.ContactFeature.CONTACT_INFO,
            Tp.ContactFeature.LOCATION,
            Tp.ContactFeature.CONTACT_BLOCKING,
        ])

        am.prepare_async(None, self._am_ready, None)

        box = elm.Box(window)

        self.contact_list = contact_list = elm.Genlist(window)
        contact_list.size_hint_weight_set(1.0, 1.0)
        contact_list.size_hint_align_set(-1.0, -1.0)
        contact_list.mode_set(elm.ELM_LIST_COMPRESS)
        box.pack_end(contact_list)
        contact_list.show()
        contact_list.callback_longpressed_add(self.create_contact_info)
        contact_list.callback_clicked_double_add(self.show_menu)
        
        btn_box = elm.Box(window)
        btn_box.horizontal_set(True)
        btn_box.size_hint_weight_set(1.0, 0.0)
        btn_box.size_hint_align_set(-1.0, 1.0)
        
        #add_btn = elm.Button(window)
        #add_btn.callback_clicked_add(self.new_contact)
        #Translators: Verb, to add something
        #add_btn.text_set(_("Add").encode("utf-8"))
        #add_btn.size_hint_weight_set(1.0, 0.0)
        #add_btn.size_hint_align_set(-1.0, 1.0)

        #btn_box.pack_end(add_btn)
        #add_btn.show()
        #block_btn.show()
        #box.pack_end(btn_box)
        #btn_box.show()

        self.pager = pager = elm.Naviframe(window)
        pager.size_hint_weight_set(1.0, 1.0)
        pager.size_hint_align_set(-1.0, -1.0)
        window.resize_object_add(pager)
        pager.show()
        pager.item_simple_push(box)

        window.resize(480, 640)
        window.show()

    def _am_ready(self, am, result, error):
        for account in am.get_valid_accounts():
            self.accounts[account.get_path_suffix()] = item = Account(self)
            account.prepare_async(None, item.account_ready, None)
            log.info("Added account for %s to the list of accounts" % (account.get_display_name()))
            
    def new_contact(self, obj, *args):
        new_contact_page = NewContact(self, self.am)

    def create_contact_info(self, genlist, item):
        contact = item.data_get()
        contact_info = ContactInfo(self, contact)
        self.pager.item_simple_push(contact_info)
        
    def start_conversation(self, genlist, item):
        contact = item.data_get()
        account = contact.get_connection().get_account()
        contact_id = contact.get_identifier()
        request_dict = {
            Tp.PROP_CHANNEL_CHANNEL_TYPE: Tp.IFACE_CHANNEL_TYPE_TEXT,
            Tp.PROP_CHANNEL_TARGET_HANDLE_TYPE: int(Tp.HandleType.CONTACT),
            Tp.PROP_CHANNEL_TARGET_ID: contact_id
        }
        request = Tp.AccountChannelRequest.new(account, request_dict, 0)
        request.ensure_channel_async("", None, self.ensure_channel_cb, None)
        
    def start_call(self, genlist, item):
        contact = item.data_get()
        account = contact.get_connection().get_account()
        contact_id = contact.get_identifier()
        request_dict = {
            Tp.PROP_CHANNEL_CHANNEL_TYPE: Tp.IFACE_CHANNEL_TYPE_CALL,
            Tp.PROP_CHANNEL_TARGET_HANDLE_TYPE: int(Tp.HandleType.CONTACT),
            Tp.PROP_CHANNEL_TARGET_ID: contact_id
        }
        request = Tp.AccountChannelRequest.new(account, request_dict, 0)
        request.ensure_channel_async("", None, self.ensure_channel_cb, None)
        
    def ensure_channel_cb(self, request, result, user_data):
        try:
            request.ensure_channel_finish(result)
        except GObject.GError as gerror:
            ErrorDialog(self.window, _("Error"), gerror.message)
            
    def block_contact(self, genlist, item):
        contact = item.data_get()
        contact.block_async(False, self.contact_blocked, None)
        
    def contact_blocked(self, contact, result, user_data):
        try:
            contact.block_finish(result)
        except GObject.GError as gerror:
            ErrorDialog(self, _("Error"), gerror.message)
            
    def unblock_contact(self, genlist, item):
        contact = item.data_get()
        contact.unblock_async(self.contact_unblocked, None)
        
    def contact_unblocked(self, contact, result, user_data):
        try:
            contact.unblock_finish(result)
        except GObject.GError as gerror:
            ErrorDialog(self, _("Error"), gerror.message)
        
    def show_menu(self, genlist, item):
        contact = item.data_get()
        menu = elm.Menu(self.window)
        menu.item_add(None, _("Text").encode("utf-8"), "message", self.menu_conversation, genlist, item)
        #menu.item_add(None, _("Call"), "call", self.menu_call, genlist, item)
        #menu.item_add(None, _("Send file", "file", self.menu_file, genlist, item)
        menu.item_separator_add()
        if contact.is_blocked():
            menu.item_add(None, _("Unblock").encode("utf-8"), "", self.menu_unblock, genlist, item)
        else:
            menu.item_add(None, _("Block").encode("utf-8"), "", self.menu_block, genlist, item)
        menu.item_add(None, _("Information").encode("utf-8"), "", self.menu_contact_info, genlist, item)
        
        w, h = genlist.size_get()
        menu.move(w/2-50, h/2-100)
        menu.show()
        
    def menu_conversation(self, menu, menuitem, genlist, item):
        self.start_conversation(genlist, item)
    def menu_call(self, menu, menuitem, genlist, item):
            self.start_call(genlist, item)
    def menu_file(self, menu, menuitem, genlist, item):
            pass
    def menu_block(self, menu, menuitem, genlist, item):
            self.block_contact(genlist, item)
    def menu_unblock(self, menu, menuitem, genlist, item):
            self.unblock_contact(genlist, item)
    def menu_contact_info(self, menu, menuitem, genlist, item):
            self.create_contact_info(genlist, item)

    def quit(self, *args):
        elm.exit()

class Account(object):
    def __init__(self, parent):
        self.parent = parent

        self._signals = []
        self.group_items = {}

    def account_ready(self, account, result, error):
        self.account = account

        if account.get_connection_status()[0] == Tp.ConnectionStatus.CONNECTED:
            self.get_contacts(account.get_connection())

        account.connect("status-changed", self._status_changed)
        
    def _status_changed(self, account, old_status, new_status, reason, dbus_error_name, details, *args):
        log.debug("Connection status for %s changed from %s to %s, reason %s, dbus error %s, details %s" % (account.get_display_name(), old_status, new_status, reason, dbus_error_name, details))
        connection = account.get_connection()
        contact_list = self.parent

        if new_status == Tp.ConnectionStatus.CONNECTED:
            if connection.get_contact_list_state() == Tp.ContactListState.SUCCESS:
                self._signals.append(connection.connect("groups-created", self._groups_created))
                self._signals.append(connection.connect("groups-removed", self._groups_removed))
                self.get_contacts(connection)
            else:
                self._signals.append(connection.connect("notify::contact-list-state", self._contact_list_state_changed))

        elif new_status == Tp.ConnectionStatus.DISCONNECTED:
            log.debug("%s disconnected" % (connection.get_object_path()))
            for sigid in self._signals:
                connection.disconnect(sigid)
            try:
                for k, v in connection.contact_items.iteritems():
                    for item in v:
                        item.delete()
                for k, v in connection.group_items.iteritems():
                    v.delete()
            except:
                pass

    def get_contacts(self, connection):
        connection.group_items = {}
        connection.contact_items = {}
        contact_list = self.parent
        groups = connection.get_contact_groups()
        if not groups:
            groups.append(_("Non-grouped contacts").encode("utf-8"))
        self._groups_created(groups)
        contacts = connection.dup_contact_list()
        self._contact_list_changed(contacts, None)

        self._signals.append(connection.connect("contact-list-changed", self._contact_list_changed))
        #connection.connect("balance-changed", self._balance_changed)
        #connection.connect("blocked-contacts-changed", self._blocked_contacts_changed)
        #connection.connect("group-renamed", self._group_renamed)

    def _groups_created(self, added):
        for group in added:
            self.account.get_connection().group_items[group] = item = self.add_group(group)
            item.select_mode_set(elm.ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY)

    def _groups_removed(self, removed):
        for group in removed:
            log.debug("Group %s was removed" % (group))

    def _contact_list_state_changed(self, connection, prop):
        if connection.get_contact_list_state() == Tp.ContactListState.SUCCESS:
            self.get_contacts(connection)

    def _send_message(self, obj, *args):
        pass

    def _call(self, obj, *args):
        pass

    def add_contact(self, contact):
        connection = contact.get_connection()
        item_class = ContactItem("default")
        contact_items = []
        groups = contact.get_contact_groups()
        if not groups:
            groups = [_("Contacts")]
        for group in groups:
            try:
                group = connection.group_items[group]
            except KeyError:
                group = None
            contact_item = self.parent.contact_list.item_append(item_class, contact, group, elm.ELM_GENLIST_ITEM_NONE)
            contact_item.tooltip_content_cb_set(self._contact_tooltip, contact)
            contact_items.append(contact_item)
        return contact_items

    def _contact_list_changed(self, added, removed, *args):
        if added:
            for contact in added:
                log.debug("contact %s added" % contact.get_handle())
                contact.get_connection().contact_items[contact.get_handle()] = items = self.add_contact(contact)
                #~ for item in items:
                    #~ if contact.get_presence_status() == "offline":
                        #~ item.disabled_set(True)
                    #~ else:
                        #~ item.disabled_set(False)
                contact.connect("presence-changed", self._presence_changed)
                #contact.connect("contact-groups-changed", self._groups_changed)
                #contact.connect("subscription-states-changed", self._subscription_states_changed)
                contact.connect("notify::avatar-file", self._contact_changed)
        if removed:
            for contact in removed:
                log.debug("contact %s removed" % contact.get_handle())
                contact.get_connection().contact_items[contact.get_handle()].delete()
                del contact

    def _contact_tooltip(self, list, item, contact):
        helplabel = elm.Label(self.parent.window)
        helplabel.size_hint_align_set(0.0, 0.0)
        helplabel.text_set("<b>Double-click</b> for contact options<br/><b>Longpress</b> for more information")
        
        avatar = elm.Photo(self.parent.window)
        avatar.size_set(100)
        avatar_file = contact.get_avatar_file()
        if avatar_file:
            path = avatar_file.get_path()
            avatar.file_set(path)

        textbox = elm.Box(self.parent.window)
        textcap = ""
        audiocap = ""
        audiovideocap = ""
        caps = contact.get_capabilities()
        if caps:
            if caps.supports_text_chats():
                textcap = _("Yes")
            if caps.supports_audio_call(Tp.HandleType.CONTACT):
                audiocap = _("Yes")
            if caps.supports_audio_video_call(Tp.HandleType.CONTACT):
                audiovideocap = _("Yes")
        states = {
            Tp.SubscriptionState.UNKNOWN: _("Unknown"),
            Tp.SubscriptionState.NO: _("No"),
            Tp.SubscriptionState.REMOVED_REMOTELY: _("Denied"),
            Tp.SubscriptionState.ASK: _("Waiting confirmation"),
            Tp.SubscriptionState.YES: _("Yes")
        }
        texts = {
            _("Identifier"): contact.get_identifier(),
            _("Status"): contact.get_presence_status().capitalize(),
            _("Message"): contact.get_presence_message().decode("utf-8"),
            _("Account"): contact.get_connection().get_account().get_display_name(),
            _("Subscribed to"): states[contact.get_subscribe_state()],
            _("Published to"): states[contact.get_publish_state()],
            _("Can send text"): textcap,
            _("Can call"): audiocap,
            _("Can call with video"): audiovideocap,
        }
        for k, v in texts.iteritems():
            k = k.encode("ascii", "xmlcharrefreplace")
            v = v.encode("ascii", "xmlcharrefreplace")
            label = elm.Label(self.parent.window)
            label.text_set("<b>%s:</b> %s" % (k, v))
            label.size_hint_align_set(0.0, 0.0)
            textbox.pack_end(label)
            label.show()
        
        infobox = elm.Box(self.parent.window)
        infobox.horizontal_set(True)
        infobox.pack_end(avatar)
        avatar.show()
        infobox.pack_end(textbox)
        textbox.show()
        box = elm.Box(self.parent.window)
        box.pack_end(helplabel)
        helplabel.show()
        box.pack_end(infobox)
        infobox.show()
        box.show()
        return box

    def add_group(self, group):
        group_class = ContactGroup()
        return self.parent.contact_list.item_append(group_class, group, None, elm.ELM_GENLIST_ITEM_GROUP)

    def _group_renamed(self, old_name, new_name, *args):
        pass

    def _presence_changed(self, contact, presence, status, message, *args):
        for item in contact.get_connection().contact_items[contact.get_handle()]:
            #~ if status == "offline":
                #~ item.disabled_set(True)
            #~ else:
                #~ item.disabled_set(False)
            item.update()
        notify(_("Presence changed"), _("%s is now %s").encode("utf-8") % (contact.get_alias(), status), "notification-information")

    def _contact_changed(self, contact, signal):
        for item in contact.get_connection().contact_items[contact.get_handle()]:
            item.update()

if __name__ == "__main__":
    gettext.install("apathy", "/usr/share/locale", unicode=1)

    # Log
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    log = logging.getLogger("apathy.contacts")
    log.setLevel(logging.DEBUG)
    streamformatter = logging.Formatter("[%(levelname)s] in %(module)s line %(lineno)s (%(funcName)s): %(message)s")
    handler_stream = logging.StreamHandler()
    handler_stream.setFormatter(streamformatter)
    log.addHandler(handler_stream)

    elm.init()

    ecore.main_loop_glib_integrate()
    contacts = Contacts()

    elm.run()
    elm.shutdown()
