#!/usr/bin/env python

#########################################################################
#                                                                       #
# Apathy - A telepathy based communications client for mobile platforms #
# Copyright (C) 2009  Michael Sheldon <mike@mikeasoft.com>              #
# Copyright 2012 Kai Huuhko <kai@kaihuuhko.com>                         #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
#                                                                       #
#########################################################################

import os.path
import gettext
import logging

import ecore
import elementary as elm
from evas import EVAS_ASPECT_CONTROL_VERTICAL

from Globals import *
from Conversation import Conversation

from gi.repository import GObject
GObject.threads_init()
from gi.repository import TelepathyGLib as Tp

class Handler(object):
    def __init__(self):
        global log
        log = logging.getLogger("apathy.handler")

        window = self.window = elm.StandardWindow("handler", "Apathy - Handler")
        window.callback_delete_request_add(self.quit)
        window.callback_focus_in_add(lambda x: set_urgent(self.window, False))

        box = elm.Box(window)
        box.size_hint_weight_set(1.0, 1.0)
        box.size_hint_align_set(-1.0, -1.0)
        self.window.resize_object_add(box)

        self.chatlist = chatlist = elm.Hoversel(window)
        self.chatlist.text_set(_("Channels"))
        chatlist.size_hint_weight_set(1.0, 0.0)
        chatlist.size_hint_align_set(-1.0, -1.0)
        box.pack_end(chatlist)
        chatlist.show()

        self.pager = pager = elm.Naviframe(window)
        pager.size_hint_weight_set(1.0, 1.0)
        pager.size_hint_align_set(-1.0, -1.0)
        #self.resize_object_add(pager)
        box.pack_end(pager)
        pager.show()

        xbtn = elm.Button(window)
        xbtn.callback_clicked_add(self.close_channel_box)
        xbtn.text_set(_("Close"))
        xbtn.size_hint_weight_set(1.0, 0.0)
        xbtn.size_hint_align_set(-1.0, -1.0)
        box.pack_end(xbtn)
        xbtn.show()

        box.show()

        daemon = Tp.DBusDaemon.dup()
        factory = Tp.AutomaticClientFactory.new(daemon)
        factory.add_account_features([
            Tp.Account.get_feature_quark_connection()
        ])
        factory.add_connection_features([
            Tp.Connection.get_feature_quark_capabilities(),
            Tp.Connection.get_feature_quark_contact_list(),
            Tp.Connection.get_feature_quark_contact_groups(),
            Tp.Connection.get_feature_quark_avatar_requirements(),
        ])
        factory.add_contact_features([
            Tp.ContactFeature.ALIAS,
            Tp.ContactFeature.CONTACT_GROUPS,
            Tp.ContactFeature.PRESENCE,
            Tp.ContactFeature.AVATAR_DATA,
            Tp.ContactFeature.SUBSCRIPTION_STATES,
            Tp.ContactFeature.CAPABILITIES,
            Tp.ContactFeature.CONTACT_INFO,
            Tp.ContactFeature.LOCATION,
        ])
        factory.add_channel_features([
            Tp.Channel.get_feature_quark_contacts(),
            Tp.Channel.get_feature_quark_chat_states(),
            Tp.TextChannel.get_feature_quark_incoming_messages(),
        ])

        self.handler = Tp.SimpleHandler.new_with_factory(
            factory,                #Factory/am/tpdbus
            False,                  #bypass approval
            False,                  #requests
            'Apathy.Handler',        #name
            False,                  #uniquify
            self.channel_handler,   #handler func
            None,                   #data
        )
        self.handler.add_handler_filter(
            {
                Tp.PROP_CHANNEL_CHANNEL_TYPE: Tp.IFACE_CHANNEL_TYPE_TEXT,
                Tp.PROP_CHANNEL_TARGET_HANDLE_TYPE: int(Tp.HandleType.CONTACT),
            }
        )
        self.handler.add_handler_capabilities([
            Tp.IFACE_CHANNEL_INTERFACE_MESSAGES,
        ])
        self.handler.register()

    def channel_handler(self, handler, account, connection, channels, requests, user_action_time, context, user_data):
        log.debug("%s %s %s %s %s %s %s %s" % (handler, account, connection, channels, requests, user_action_time, context, user_data))
        for c in channels:
            log.debug(c.get_handle())

        box_of_channels = Channels(self, channels)
        box_of_channels.show()

        identifier = channels[0].get_target_contact().get_alias()
        if not identifier:
            identifier = channels[0].get_identifier()

        try:
            avatar = channels[0].get_target_contact().get_avatar_file().get_path()
        except:
            avatar = None

        box_of_channels.list_item = self.chatlist.item_add(identifier, avatar, elm.ELM_ICON_FILE, self.selected, box_of_channels)

        self.pager.item_simple_push(box_of_channels)
        self.window.resize(480, 640)
        self.window.show()

        context.accept()

    def selected(self, chatlist, item, pager_item):
        self.pager.item_simple_promote(pager_item)

    def close_channel_box(self, obj, *args):
        box_of_channels = self.pager.top_item_get().content_get()
        try:
            for channel in box_of_channels.channels:
                channel.close_async(lambda x, y, z: None, None)
            self.pager.item_pop()
            box_of_channels.list_item.delete()
            del box_of_channels
        except:
            pass
        if not self.pager.top_item_get():
            self.quit()

    def quit(self, *args):
        elm.exit()

class Channels(elm.Box):
    def __init__(self, parent, channels):
        elm.Box.__init__(self, parent.window)
        self.channels = []
        self.list_item = None

        self.size_hint_weight_set(1.0, 1.0)
        self.size_hint_align_set(-1.0, -1.0)

        for channel in channels:
            if isinstance(channel, Tp.TextChannel):
                log.info("Handling text channel with %s" % (channel.get_identifier()))
                conversation = Conversation(parent, channel)
                self.pack_end(conversation.box)
                self.channels.append(channel)
            else:
                log.error("Non-supported channel")

if __name__ == "__main__":
    gettext.install("apathy", "/usr/share/locale", unicode=1)

    # Log
    if not os.path.exists(USER_PATH):
        os.mkdir(USER_PATH, 0700)
    log = logging.getLogger("apathy.handler")
    log.setLevel(logging.DEBUG)
    streamformatter = logging.Formatter("[%(levelname)s] in %(module)s line %(lineno)s (%(funcName)s): %(message)s")
    handler_stream = logging.StreamHandler()
    handler_stream.setFormatter(streamformatter)
    log.addHandler(handler_stream)

    elm.init()

    ecore.main_loop_glib_integrate()
    handler = Handler()

    elm.run()
    elm.shutdown()
